# _entity.py
#
# openipmi GUI handling for entities
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import OpenIPMI
import _oi_logging
import _sensor
import _control
import _fru
import gui_popup
import gui_setdialog

class EntityOp:
    def __init__(self, e, func):
        self.e = e
        self.func = func
        return

    def DoOp(self):
        rv = self.e.entity_id.to_entity(self)
        if (rv == 0):
            rv = self.rv
            pass
        return rv;

    def entity_cb(self, entity):
        self.rv = getattr(entity, self.func)(self.e)
        return

    pass

class EntityFruViewer:
    def __init__(self, e):
        self.e = e
        self.e.entity_id.to_entity(self)
        return

    def entity_cb(self, entity):
        fru = entity.get_fru()
        if (fru == None):
            return
        _fru.FruInfoDisplay(fru, str(self.e))
        return

    pass

class ActivationTimeSetter:
    def __init__(self, e, name, func):
        self.e = e
        self.name = name
        self.func = func

        self.err = 0
        rv = e.entity_id.to_entity(self)
        if (rv == 0):
            rv = self.err
            pass
        if (rv):
            _oi_logging.error("Error doing entity cb in activation time setter: "
                          + str(rv))
            pass

        return

    def ok(self, val):
        self.acttime = int(val)
        rv = self.e.entity_id.to_entity(self)
        if (rv == 0):
            rv = self.err
            pass
        if (rv != 0):
            return ("Error setting activation time: "
                    + OpenIPMI.get_error_string(rv))
        return

    def entity_cb(self, entity):
        if (self.setting):
            self.err = getattr(entity, 'set_' + self.func)(None)
        else:
            self.err = getattr(entity, 'get_' + self.func)(self)
            pass
        return

    def entity_hot_swap_get_time_cb(self, entity, err, time):
        if (err):
            _oi_logging.error("Error getting activation time: " + str(err))
            return
        gui_setdialog.SetDialog("Set " + self.name + " for " + str(self.e),
                                [ time ],
                                1,
                                self)
        return

    pass

class Entity:
    def __init__(self, d, entity):
        self.d = d
        self.ui = d.ui
        self.name = entity.get_name()
        self.entity_id = entity.get_id()
        self.entity_id_str = entity.get_entity_id_string()
        self.sensors = { }
        self.controls = { }
        self.children = { }
        entity.add_presence_handler(self)
        entity.add_hot_swap_handler(self)
        entity.add_sensor_update_handler(self)
        entity.add_control_update_handler(self)

        d.entities[self.name] = self

        self.destroyed = False

        # Find my parent and put myself in that hierarchy
        self.parent_id = None
        self.parent = None
        self.eeop = "fparent"
        entity.iterate_parents(self)
        self.ui.add_entity(self.d, self, parent=self.parent)
        if (self.parent != None):
            self.parent.children[self.name] = self.name
            self.parent_name = self.parent.name
            pass
        else:
            self.parent_name = None
            pass
        self.parent = None # Don't leave circular reference

        self.hot_swap = 'No'
        self.hot_swap_state = ''
        self.create_my_items()

        self.Changed(entity)
        return

    def create_my_items(self):
        self.ui.append_item(self, 'Entity ID', self.entity_id_str)
        self.typeitem = self.ui.append_item(self, 'Type', None)
        self.idstringitem = self.ui.append_item(self, 'ID String', None)
        self.psatitem = self.ui.append_item(self,
                                            'Presence Sensor Always There',
                                            None)
        self.slotnumitem = self.ui.append_item(self, 'Slot Number', None)
        self.mcitem = self.ui.append_item(self, 'MC', None)
        self.hotswapitem = self.ui.append_item(self, 'Hot Swap', self.hot_swap)
        return
    
    def __str__(self):
        return self.name

    def HandleMenu(self, event):
        if (self.impt_data == None):
            menul = [ [ "Add to watch values", self.add_impt ] ]
            pass
        else:
            menul = [ [ "Remove from watch values", self.remove_impt ] ]
            pass
        if (self.is_fru):
            menul.append(["View FRU Data", self.ViewFruData])
            pass
            
        if (self.hot_swap == "Managed"):
            menul.append(["Request Activation", self.RequestActivation])
            menul.append(["Activate", self.Activate])
            menul.append(["Deactivate", self.Deactivate])
            if (self.supports_auto_activate):
                menul.append(["Set Auto-activate Time", self.SetAutoActTime])
                pass
            if (self.supports_auto_deactivate):
                menul.append(["Set Auto-deactivate Time",
                              self.SetAutoDeactTime])
                pass
            pass

        if (len(menul) > 0):
            gui_popup.popup(self.d.ui, event, menul)
            pass
        return

    def add_impt(self, event):
        self.ui.add_impt_data("entity", str(self), self)
        return
        
    def remove_impt(self, event):
        self.ui.remove_impt_data(self.impt_data)
        return
        
    def RequestActivation(self, event):
        oper = EntityOp(self, "set_activation_requested")
        rv = oper.DoOp()
        if (rv != 0):
            _oi_logging.error("entity set activation failed: " + str(rv))
            pass
        return

    def Activate(self, event):
        oper = EntityOp(self, "activate")
        rv = oper.DoOp()
        if (rv != 0):
            _oi_logging.error("entity activation failed: " + str(rv))
            pass
        return

    def Deactivate(self, event):
        oper = EntityOp(self, "deactivate")
        rv = oper.DoOp()
        if (rv != 0):
            _oi_logging.error("entity deactivation failed: " + str(rv))
            pass
        return

    def SetAutoActTime(self, event):
        ActivationTimeSetter(self, "Activation Time", "auto_activate_time")
        return
        
    def SetAutoDeactTime(self, event):
        ActivationTimeSetter(self, "Deactivation Time", "auto_deactivate_time")
        return
        
    def ViewFruData(self, event):
        EntityFruViewer(self)
        return

    def entity_activate_cb(self, entity, err):
        if (err != 0):
            _oi_logging.error("entity activate operation failed: " + str(err))
            pass
        return
        
    def Changed(self, entity):
        # Reparent first, in case parent has changed.
        old_parent_id = self.parent_id
        reparent = False
        self.parent_id = None
        self.parent = None
        self.eeop = "fparent"
        entity.iterate_parents(self)
        if (self.parent_id != None):
            if (old_parent_id == None):
                reparent = True
                pass
            elif (self.parent_id.cmp(old_parent_id) != 0):
                reparent = True
                pass
            pass
        else:
            if (old_parent_id != None):
                reparent = True
                pass
            pass

        if (reparent):
            old_treeroot = self.treeroot
            self.eop = "deleted"
            entity.iterate_sensors(self)
            entity.iterate_controls(self)

            if (self.parent_name):
                oparent = self.d.find_entity_byname(self.parent_name)
                if (oparent):
                    del oparent.children[self.name]
                pass
            self.ui.reparent_entity(self.d, self, self.parent)
            if (self.parent != None):
                self.parent.children[self.name] = self.name
                self.parent_name = self.parent.name
                pass
            else:
                self.parent_name = None
                pass
            self.create_my_items()

            # Reparent children
            self.eeop = "repch"
            entity.iterate_children(self);
            
            self.eop = "added"
            entity.iterate_sensors(self)
            entity.iterate_controls(self)

            if (entity.is_present()):
                self.ui.set_item_active(self.treeroot);
                pass
            else:
                self.ui.set_item_inactive(self.treeroot);
                pass

            self.eop = None
            self.ui.remove_entity(old_treeroot)
            pass
        
        self.parent = None # Kill circular reference


        self.is_fru = entity.is_fru()
        
        self.id_str = entity.get_id_string()
        eid = self.id_str
        if (eid == None):
            eid = self.entity_id_str
            pass
        if (eid != None):
            self.ui.set_item_text(self.treeroot, eid)
            pass

        self.entity_type = entity.get_type()
        self.slot_number = entity.get_physical_slot_num()
        if (self.slot_number < 0):
            self.slot_number = None
            pass

        self.mc_id = entity.get_mc_id()
        self.mc_name = None
        if (self.mc_id != None):
            self.mc_id.to_mc(self);
            pass

        if entity.is_hot_swappable():
            if entity.supports_managed_hot_swap():
                hot_swap = "Managed"
            else:
                hot_swap = "Simple"
                pass
        else:
            hot_swap = "No"
            pass
        if (hot_swap != self.hot_swap):
            self.hot_swap = hot_swap
            if (hot_swap != "No"):
                entity.get_hot_swap_state(self)
            else:
                self.hot_swap_state = ''
                pass
            pass

        self.supports_auto_activate = entity.supports_auto_activate_time()
        self.supports_auto_deactivate = entity.supports_auto_deactivate_time()

        # Fill in the various value in the GUI
        self.ui.set_item_text(self.typeitem, self.entity_type)
        self.ui.set_item_text(self.idstringitem, self.id_str)
        self.ui.set_item_text(self.psatitem,
                           str(entity.get_presence_sensor_always_there() != 0))
        if (self.slot_number != None):
            self.ui.set_item_text(self.slotnumitem, str(self.slot_number))
            pass
        self.ui.set_item_text(self.mcitem, self.mc_name)
        self.ui.set_item_text(self.hotswapitem,
                              self.hot_swap + ' ' + self.hot_swap_state)
        return

    def entity_hot_swap_update_cb(self, entity, old_state, new_state, event):
        if (self.destroyed):
            return
        self.hot_swap_state = new_state
        self.ui.set_item_text(self.hotswapitem,
                              self.hot_swap + ' ' + self.hot_swap_state)
        return OpenIPMI.EVENT_NOT_HANDLED
    
    def entity_hot_swap_cb(self, entity, err, state):
        if (self.destroyed):
            return
        if (err):
            _oi_logging.error("Error getting entity hot-swap state: " + str(err))
            return
        self.hot_swap_state = state
        self.ui.set_item_text(self.hotswapitem,
                              self.hot_swap + ' ' + self.hot_swap_state)
        return
    
    def mc_cb(self, mc):
        self.mc_name = mc.get_name()
        return
        
    def entity_iter_entities_cb(self, e1, e2):
        if (self.eeop == "fparent"):
            self.parent_id = e2.get_id()
            self.parent = self.d.find_or_create_entity(e2)
            pass
        elif (self.eeop == "repch"):
            ch_e = self.d.find_entity_byname(e2.get_name())
            if (ch_e):
                ch_e.parent_id = None
                ch_e.Changed(e2)
                pass
            pass
        return

    def entity_iter_sensors_cb(self, entity, sensor):
        self.entity_sensor_update_cb(self.eop, entity, sensor)
        return
    
    def entity_iter_controls_cb(self, entity, control):
        self.entity_control_update_cb(self.eop, entity, control)
        return
    
    def remove(self):
        self.d.entities.pop(self.name)
        self.ui.remove_entity(self)
        self.destroyed = True
        self.d = None
        self.ui = None
        return

    def entity_sensor_update_cb(self, op, entity, sensor):
        if (self.destroyed):
            return
        if (op == "added"):
            if (sensor.get_name() not in self.sensors):
                e = _sensor.Sensor(self, sensor)
                pass
            pass
        elif (op == "deleted"):
            if (sensor.get_name() in self.sensors):
                self.sensors[sensor.get_name()].remove()
                pass
            pass
        return

    def entity_control_update_cb(self, op, entity, control):
        if (self.destroyed):
            return
        if (op == "added"):
            if (control.get_name() not in self.controls):
                e = _control.Control(self, control)
                pass
            pass
        elif (op == "deleted"):
            if (control.get_name() in self.controls):
                self.controls[control.get_name()].remove()
                pass
            pass
        return

    def entity_presence_cb(self, entity, present, event):
        if (self.destroyed):
            return
        if (present):
            self.ui.set_item_active(self.treeroot)
        else:
            self.ui.set_item_inactive(self.treeroot)
            pass
        return

    pass



# gui_errstr.py
#
# Error display string
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

import Tix

class ErrStr(Tix.Label):
    def __init__(self, parent):
        Tix.Label.__init__(self, parent, relief="ridge", bd=5)
        return

    def SetError(self, text):
        self.config(text=text)
        return

    pass



#!/usr/bin/env python

# openipmigui.py
#
# The openipmi GUI startup file
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2005 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
# This is a GUI interface to the OpenIPMI library.
#
# Naming Conventions:
#
#  The following names are used universally throughout this program.
#  Note that you have two versions of each object, the Python object
#  (class is defined in this program) and the OpenIPMI version of the
#  object (defined by the OpenIPMI library).
#
#  domain - OpenIPMI.ipmi_domain_t
#  domain_id - OpenIPMI.ipmi_domain_id_t
#  entity - OpenIPMI.ipmi_entity_t
#  entity_id - OpenIPMI.ipmi_entity_id_t
#  mc - OpenIPMI.ipmi_mc_t
#  mc_id - OpenIPMI.ipmi_mc_id_t
#  sensor - OpenIPMI.ipmi_sensor_t
#  sensor_id - OpenIPMI.ipmi_sensor_id_t
#  control - OpenIPMI.ipmi_control_t
#  control_id - OpenIPMI.ipmi_control_id_t
#
#  d - Domain
#  e - Entity
#  m - MC
#  s - Sensor
#  c - Control

import os
import Tix
import sys
import OpenIPMI
from openipmigui import _domain
from openipmigui import gui
from openipmigui import _saveprefs
from openipmigui import gui_cmdwin

# Used to enable internal debug output
verbosity = 0

#
# Nasty thread support stuff.
#
# If python is threaded but TCL is not, we have ugliness to deal with.
# In this case, the _tkinter code has a tcl_lock that is claimed
# whenever it enters TCL code, including the event loop.  If we run
# OpenIPMI events inside the TCL event loop, it will call directly
# into the OpenIPMI calls and bypass _tkinter, leaving the lock held.
# Inside OpenIPMI, it might call back to the python code which can
# then do a TCL call, which will try to claim tcl_lock and deadlock.
#
# We fix this in this case by having a separate thread
# (openipmi_driver() below) run OpenIPMI events, and not in the TCL
# event loop.
#
# If both python and TCL are threaded, the tcl_lock is not used and
# everything is fine with a single thread.  If python is not threaded,
# then threads don't matter.  In those cases we use the TCL event loop
# for everything.
#
# See the _tkinter.c file in the python distribution for more details.
#
shutdown_thread = False
def openipmi_driver():
    while (not shutdown_thread):
        OpenIPMI.wait_io(1000)
        pass
    return

class TopHandler(Tix.Tk):
    def __init__(self, preffile, log_file, histfile):

        Tix.Tk.__init__(self)
        
        self.domains = { };
        self.preffile = preffile
        self.log_file = log_file
        self.histfile = histfile
        return

    def domain_change_cb(self, op, domain):
        if (op == "added"):
            self.domains[domain.get_name()].connected(domain)
        elif (op == "removed"):
            self.domains[domain.get_name()].remove()
            pass
        return

    def SetUI(self, ui):
        self.ui = ui;
        return

    def savePrefs(self):
        objs = self.domains.values()
        objs.append(self.ui)
        _saveprefs.save(objs, self.preffile)
        return

    def log(self, level, log):
        if (self.log_file != None):
            self.log_file.write(level + ": " + log + "\n")
        self.ui.new_log(level + ": " + log)
        return

    def quit(self):
        global shutdown_thread
        shutdown_thread = True;
        gui_cmdwin._HistorySave(self.histfile)

        OpenIPMI.set_log_handler(DummyLogHandler())
        OpenIPMI.shutdown_everything()
        if (self.debug_mem):
            print "OpenIPMI is shutdown, memory problems (SEGVs) after this"
            print " are likely due to OpenIPMI data not being freed until"
            print " after this point due to the python garbage collector"
            pass
        sys.exit()
        return

class DummyLogHandler:
    def __init__(self):
        pass

    def log(self, level, log):
        sys.stderr.write(level + ": " + log + "\n")

class CmdlangEventHandler:
    def __init__(self, app):
        self.app = app
        return
    
    def cmdlang_event(self, event):
        if (not self.app.ui.logevents):
            return
        name = [ "" ]
        value = [ "" ]
        vtype = [ "" ]
        level = [ 0 ]
        event.restart()
        estr = "Event:"
        more = event.next_field(level, vtype, name, value)
        while (more != 0):
            estr += "\n  %*s%s: %s" % (level[0], "", name[0], value[0])
            more = event.next_field(level, vtype, name, value)
            pass
        self.app.ui.new_log(estr)
        return
    
    pass

def trace(frame, event, arg):
    print (event + ": " + frame.f_code.co_name +
           "(" + frame.f_code.co_filename + ":" + str(frame.f_lineno) + ")")
    return trace

def run(args):
    global verbosity

    preffile = os.path.join(os.environ['HOME'], '.openipmigui.startup')
    histfile = os.path.join(os.environ['HOME'], '.openipmigui.history')

    debug_msg = False
    debug_rawmsg = False
    debug_mem = False
    do_trace = False
    read_preffile = True
    log_file = None

    # Skip program name.
    carg = 1

    while (carg < len(args)):
        arg = args[carg]
        carg += 1
        if (arg == "--dmsg"):
            debug_msg = True
        elif (arg == "--drawmsg"):
            debug_msg = True
        elif (arg == "--dmem"):
            debug_mem = True
        elif (arg == "--verbose"):
            verbosity += 1
        elif (arg == "--trace"):
            do_trace = True
        elif (arg == "--logstderr"):
            log_file = sys.stderr
        elif (arg == "--logstdout"):
            log_file = sys.stdout
        elif (arg == "-n"):
            read_preffile = False
        elif (arg == '-p'):
            if (len(args) == 0):
                print "No argument given for -p";
                return
            preffile = args[carg]
            carg += 1
        else:
            print "Unknown argument: " + arg
            return
        pass

    if (debug_mem):
        OpenIPMI.enable_debug_malloc()
        pass

    top = TopHandler(preffile, log_file, histfile)

    # Detect if we need a separate OpenIPMI driver thread.  See the
    # openipmi_driver function above for the reason.
    try:
        import thread
        try:
            top.tk.getvar("tcl_platform", "threaded")
            # Tcl is threaded, no need for another thread.
            need_separate_openipmi_thread = False
        except:
            # Python is threaded, but Tcl is not.  Need to run the
            # OpenIPMI event loop in another thread.
            need_separate_openipmi_thread = True
            pass
        pass
    except:
        # No thread support, can't use another thread.
        need_separate_openipmi_thread = False
        pass

    if (need_separate_openipmi_thread):
        if (verbosity >= 1):
            print "Creating separate OpenIPMI event driver thread"
            pass
        OpenIPMI.init()
        thread.start_new_thread(openipmi_driver, ())
        pass
    else:
        if (verbosity >= 1):
            print "Using TCL event loop, no threads"
            pass
        OpenIPMI.init_tcl()
        pass

    if (debug_rawmsg):
        OpenIPMI.enable_debug_rawmsg()
        pass
    if (debug_msg):
        OpenIPMI.enable_debug_msg()
        pass
    if (do_trace):
        sys.settrace(trace)
        pass

    if (read_preffile):
        _saveprefs.restore(preffile)
    gui_cmdwin._HistoryRestore(histfile)
    
    mainhandler = top

    OpenIPMI.add_domain_change_handler(_domain.DomainWatcher(mainhandler))

    mainhandler.debug_mem = debug_mem
    top.title('OpenIPMI GUI')

    ui = gui.IPMIGUI(top, mainhandler)
    mainhandler.SetUI(ui)

    OpenIPMI.add_domain_change_handler(mainhandler)
    OpenIPMI.set_log_handler(mainhandler)

    _domain.RestoreDomains(mainhandler)

    OpenIPMI.set_cmdlang_event_handler(CmdlangEventHandler(mainhandler))

    top.mainloop()

    mainhandler.quit()


if __name__ == "__main__":
    run(sys.argv)



# _SoL.py
#
# openipmi GUI SoL handling
#
# Author: MontaVista Software, Inc.
#         Corey Minyard <minyard@mvista.com>
#         source@mvista.com
#
# Copyright 2006 MontaVista Software Inc.
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public License
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#
#
#  THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
#  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
#  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
#  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
#  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
#  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
#  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
#  USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this program; if not, write to the Free
#  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
import OpenIPMI
import Tix
import gui_setdialog
import gui_term
import gui_errstr

# Note in this file SoL refers to the main SoL object and sol refers
# to the connection.

class AckTimeoutSet:
    def __init__(self, SoL):
        self.SoL = SoL
        gui_setdialog.SetDialog("Set Ack Timeout for SoL",
                                [ self.SoL.sol.get_ACK_timeout() ], 1, self,
                                [ "Value (in microseconds)"] )
        return

    def do_on_close(self):
        self.SoL = None
        return
    
    def ok(self, vals):
        if (self.SoL.sol):
            self.SoL.sol.set_ACK_timeout(int(vals[0]))
        return

    pass


class AckRetriesSet:
    def __init__(self, SoL):
        self.SoL = SoL
        gui_setdialog.SetDialog("Set Ack Retries for SoL",
                                [ self.SoL.sol.get_ACK_retries() ], 1, self,
                                [ "Value (in microseconds)"] )
        return

    def do_on_close(self):
        self.SoL = None
        return
    
    def ok(self, vals):
        if (self.SoL.sol):
            self.SoL.sol.set_ACK_retries(int(vals[0]))
        return

    pass


class SolTerm(gui_term.Terminal):
    def __init__(self, parent, SoL):
        self.SoL = SoL
        gui_term.Terminal.__init__(self, parent)
        return

    def HandleTerminalOutput(self, data):
        self.SoL.HandleOutput(data);
        return

class SoL(Tix.Toplevel):
    def __init__(self, ui, domain_id, cnum):
        Tix.Toplevel.__init__(self)
        self.ui = ui
        self.cnum = cnum;
        self.sol = None
        domain_id.to_domain(self)
        if (self.sol == None):
            return
        self.nack_count = 0
        self.take_input = True
        self.in_destroy = False
        self.title("SoL for " + self.dname + " connection "
                   + str(self.cnum))

        mbar = Tix.Frame(self)
        fileb = Tix.Menubutton(mbar, text="File", underline=0, takefocus=0)
        filemenu = Tix.Menu(fileb, tearoff=0)
        self.filemenu = filemenu
        fileb["menu"] = filemenu
        filemenu.add_command(label="Open", command=self.open)
        filemenu.add_command(label="Close", command=self.close,
                             state="disabled")
        filemenu.add_command(label="Force Close", command=self.forceclose,
                             state="disabled")
        filemenu.add_command(label="Quit", command=self.quit)
        
        ctrlb = Tix.Menubutton(mbar, text="Controls", underline=0, takefocus=0)
        ctrlmenu = Tix.Menu(ctrlb, tearoff=0)
        self.ctrlmenu = ctrlmenu
        ctrlb["menu"] = ctrlmenu

        self.acceptinput = Tix.BooleanVar()
        self.acceptinput.set(True)
        ctrlmenu.add_checkbutton(label="Accept Input",
                                 variable=self.acceptinput,
                                 command=self.AcceptInputToggle)

        self.useenc = Tix.BooleanVar()
        self.useenc.set(self.sol.get_use_encryption())
        ctrlmenu.add_checkbutton(label="Use Encryption",
                                 variable=self.useenc,
                                 command=self.UseEncToggle)

        self.useauth = Tix.BooleanVar()
        self.useauth.set(self.sol.get_use_authentication())
        ctrlmenu.add_checkbutton(label="Use Authentication",
                                 variable=self.useauth,
                                 command=self.UseAuthToggle)

        self.deassert_on_connect = Tix.BooleanVar()
        self.deassert_on_connect.set(
            self.sol.get_deassert_CTS_DCD_DSR_on_connect())
        ctrlmenu.add_checkbutton(label="Deassert CTS/DCD/DSR on connect",
                                 variable=self.deassert_on_connect,
                                 command=self.DeassertOnConnectToggle)

        self.ctsassertable = Tix.BooleanVar()
        self.ctsassertable.set(True)
        ctrlmenu.add_checkbutton(label="CTS Assertable",
                                 variable=self.ctsassertable,
                                 command=self.CTSAssertableToggle,
                                 state="disabled")
        
        self.dcd_dsr = Tix.BooleanVar()
        self.dcd_dsr.set(True)
        ctrlmenu.add_checkbutton(label="DCD/DSR Asserted",
                                 variable=self.dcd_dsr,
                                 command=self.DCDDSRToggle,
                                 state="disabled")
        
        self.ri = Tix.BooleanVar()
        self.ri.set(False)
        ctrlmenu.add_checkbutton(label="RI Asserted",
                                 variable=self.ri,
                                 command=self.RIToggle,
                                 state="disabled")
        
        ctrlmenu.add_command(label="Set Ack Timeout",
                             command=self.SetAckTimeout)
        ctrlmenu.add_command(label="Set Ack Retries",
                             command=self.SetAckRetries)
        ctrlmenu.add_command(label="Send Break",
                             command=self.SendBreak, state="disabled")

        sermenu = Tix.Menu(ctrlmenu, tearoff=0)
        ctrlmenu.add_cascade(label="Serial Rate", menu=sermenu)
        self.servar = Tix.StringVar()
        self.servar.set("default")
        sermenu.add_radiobutton(label="Default", value="default",
                                variable=self.servar, command=self.SetRate)
        sermenu.add_radiobutton(label="9600", value="9600",
                                variable=self.servar, command=self.SetRate)
        sermenu.add_radiobutton(label="19200", value="19200",
                                variable=self.servar, command=self.SetRate)
        sermenu.add_radiobutton(label="38400", value="38400",
                                variable=self.servar, command=self.SetRate)
        sermenu.add_radiobutton(label="57600", value="57600",
                                variable=self.servar, command=self.SetRate)
        sermenu.add_radiobutton(label="115200", value="115200",
                                variable=self.servar, command=self.SetRate)

        serbehavemenu = Tix.Menu(ctrlmenu, tearoff=0)
        ctrlmenu.add_cascade(label="Serial Alert Behavior", menu=serbehavemenu)
        self.serbehave = Tix.StringVar()
        self.serbehave.set("fail")
        serbehavemenu.add_radiobutton(label="Serial Alerts Fail", value="fail",
                                      variable=self.serbehave,
                                      command=self.SetSerialAlerts)
        serbehavemenu.add_radiobutton(label="Serial Alerts Deferred",
                                      value="defer",
                                      variable=self.serbehave,
                                      command=self.SetSerialAlerts)
        serbehavemenu.add_radiobutton(label="Serial Alerts Succeed",
                                      value="succeed",
                                      variable=self.serbehave,
                                      command=self.SetSerialAlerts)

        flushmenu = Tix.Menu(ctrlmenu, tearoff=0)
        self.flushmenu = flushmenu
        ctrlmenu.add_cascade(label="Queue Flush", menu=flushmenu)
        fmenus = [ ]
        flushmenu.add_command(label="Flush BMC Transmit Queue",
                              command=self.FlushBMCXmit,
                              state="disabled")
        fmenus.append("Flush BMC Transmit Queue")
        flushmenu.add_command(label="Flush BMC Receive Queue",
                              command=self.FlushBMCRecv,
                              state="disabled")
        fmenus.append("Flush BMC Receive Queue")
        flushmenu.add_command(label="Flush My Transmit Queue",
                              command=self.FlushMyXmit,
                              state="disabled")
        fmenus.append("Flush My Transmit Queue")
        flushmenu.add_command(label="Flush My Receive Queue",
                              command=self.FlushMyRecv,
                              state="disabled")
        fmenus.append("Flush My Receive Queue")
        flushmenu.add_command(label="Flush BMC Queues",
                              command=self.FlushBMC,
                              state="disabled")
        fmenus.append("Flush BMC Queues")
        flushmenu.add_command(label="Flush My Queues",
                              command=self.FlushMe,
                              state="disabled")
        fmenus.append("Flush My Queues")
        flushmenu.add_command(label="Flush All Queues",
                              command=self.FlushAll,
                              state="disabled")
        fmenus.append("Flush All Queues")
        self.fmenus = fmenus

        mbar.pack(side=Tix.TOP, fill=Tix.X, expand=1)
        fileb.pack(side=Tix.LEFT)
        ctrlb.pack(side=Tix.LEFT)

        self.term = SolTerm(self, self)
        f = Tix.Frame(self)
        f.pack(side=Tix.BOTTOM, fill=Tix.X, expand=1)
        self.errstr = gui_errstr.ErrStr(f)
        self.errstr.pack(side=Tix.LEFT, fill=Tix.X, expand=1)
        self.statestr = gui_errstr.ErrStr(f)
        self.statestr.pack(side=Tix.LEFT, fill=Tix.X, expand=1)

        self.statestr.SetError(OpenIPMI.sol_state_string(
            OpenIPMI.sol_state_closed))
        self.state = OpenIPMI.sol_state_closed

        self.bind("<Destroy>", self.OnDestroy)
        return

    def OnDestroy(self, event):
        self.in_destroy = True
        if (self.sol != None):
            self.sol.force_close()
            self.sol = None
            pass
        return

    def domain_cb(self, domain):
        self.sol = domain.create_sol(self.cnum, self)
        if (self.sol == None):
            self.ui.ReportError("Unable to open SoL connection")
            self.destroy()
            return
        self.dname = domain.get_name()
        return
            
    def HandleOutput(self, data):
        self.sol.write(data)
        return

    def sol_connection_state_change(self, conn, state, err):
        if (self.in_destroy):
            return
        if (err != 0):
            self.errstr.SetError("Connection change: "
                                 + OpenIPMI.sol_state_string(state)
                                 + " " + OpenIPMI.get_error_string(err))
            pass
        self.statestr.SetError(OpenIPMI.sol_state_string(state))
        if ((self.state != OpenIPMI.sol_state_closed)
            and (state == OpenIPMI.sol_state_closed)):
            self.filemenu.entryconfigure("Open", state="normal")
            self.filemenu.entryconfigure("Close", state="disabled")
            self.filemenu.entryconfigure("Force Close", state="disabled")
            self.ctrlmenu.entryconfigure("Use Encryption", state="normal")
            self.ctrlmenu.entryconfigure("Use Authentication", state="normal")
            self.ctrlmenu.entryconfigure("Deassert CTS/DCD/DSR on connect",
                                         state="normal")
            self.ctrlmenu.entryconfigure("CTS Assertable", state="disabled")
            self.ctrlmenu.entryconfigure("DCD/DSR Asserted", state="disabled")
            self.ctrlmenu.entryconfigure("RI Asserted", state="disabled")
            self.ctrlmenu.entryconfigure("Send Break", state="disabled")
            for f in self.fmenus:
                self.flushmenu.entryconfigure(f, state="disabled")
                pass
            pass
        elif ((self.state == OpenIPMI.sol_state_closed)
            and (state != OpenIPMI.sol_state_closed)):
            self.filemenu.entryconfigure("Open", state="disabled")
            self.filemenu.entryconfigure("Close", state="normal")
            self.filemenu.entryconfigure("Force Close", state="normal")
            self.ctrlmenu.entryconfigure("Use Encryption", state="disabled")
            self.ctrlmenu.entryconfigure("Use Authentication",
                                         state="disabled")
            self.ctrlmenu.entryconfigure("Deassert CTS/DCD/DSR on connect",
                                         state="disabled")
            self.ctrlmenu.entryconfigure("CTS Assertable", state="normal")
            self.ctrlmenu.entryconfigure("DCD/DSR Asserted", state="normal")
            self.ctrlmenu.entryconfigure("RI Asserted", state="normal")
            self.ctrlmenu.entryconfigure("Send Break", state="normal")
            for f in self.fmenus:
                self.flushmenu.entryconfigure(f, state="normal")
                pass
            pass
        self.state = state
        return

    def open(self):
        self.sol.open()
        return
    
    def close(self):
        self.sol.close()
        return
    
    def forceclose(self):
        self.sol.force_close()
        return
    
    def quit(self):
        self.destroy()
        return

    def AcceptInputToggle(self):
        if (self.acceptinput.get()):
            self.take_input = True
            for i in range(0, self.nack_count):
                self.sol.release_nack();
                pass
            self.nack_count = 0;
            pass
        else:
            self.take_input = False
            pass
        return

    def UseEncToggle(self):
        self.sol.set_use_encryption(self.useenc.get())
        return
    
    def UseAuthToggle(self):
        self.sol.set_use_authentication(self.useauth.get())
        return
    
    def DeassertOnConnectToggle(self):
        self.sol.set_deassert_CTS_DCD_DSR_on_connect(
            self.deassert_on_connect.get())
        return
    
    def CTSAssertableToggle(self):
        self.sol.set_CTS_assertable(self.ctsassertable.get())
        return
    
    def DCDDSRToggle(self):
        self.sol.set_DCD_DSR_asserted(self.dcd_dsr.get())
        return
    
    def RIToggle(self):
        self.sol.set_RI_asserted(self.ri.get())
        return
    
    def SetAckTimeout(self):
        AckTimeoutSet(self)
        return

    def SetAckRetries(self):
        AckRetriesSet(self)
        return

    def SetRate(self):
        val = self.servar.get()
        if (val == "default"):
            self.sol.set_bit_rate(OpenIPMI.SOL_BIT_RATE_DEFAULT)
        elif (val == "9600"):
            self.sol.set_bit_rate(OpenIPMI.SOL_BIT_RATE_9600)
        elif (val == "19200"):
            self.sol.set_bit_rate(OpenIPMI.SOL_BIT_RATE_19200)
        elif (val == "38400"):
            self.sol.set_bit_rate(OpenIPMI.SOL_BIT_RATE_38400)
        elif (val == "57600"):
            self.sol.set_bit_rate(OpenIPMI.SOL_BIT_RATE_57600)
        elif (val == "115200"):
            self.sol.set_bit_rate(OpenIPMI.SOL_BIT_RATE_115200)
            pass
        return

    def SetSerialAlerts(self):
        val = self.serbehave.get()
        if (val == "fail"):
            self.sol.set_shared_serial_alert_behavior(
                OpenIPMI.sol_serial_alerts_fail)
        elif (val == "deferred"):
            self.sol.set_shared_serial_alert_behavior(
                OpenIPMI.sol_serial_alerts_deferred)
        elif (val == "succeed"):
            self.sol.set_shared_serial_alert_behavior(
                OpenIPMI.sol_serial_alerts_succeed)
        return

    def FlushBMCXmit(self):
        self.sol.flush(OpenIPMI.SOL_BMC_TRANSMIT_QUEUE)
        return

    def FlushBMCRecv(self):
        self.sol.flush(OpenIPMI.SOL_BMC_RECEIVE_QUEUE)
        return

    def FlushMyXmit(self):
        self.sol.flush(OpenIPMI.SOL_MANAGEMENT_CONSOLE_TRANSMIT_QUEUE)
        return

    def FlushMyRecv(self):
        self.sol.flush(OpenIPMI.SOL_MANAGEMENT_CONSOLE_RECEIVE_QUEUE)
        return

    def FlushBMC(self):
        self.sol.flush(OpenIPMI.SOL_BMC_QUEUES)
        return

    def FlushMe(self):
        self.sol.flush(OpenIPMI.SOL_MANAGEMENT_CONSOLE_QUEUES)
        return

    def FlushAll(self):
        self.sol.flush(OpenIPMI.SOL_ALL_QUEUES)
        return

    def SendBreak(self):
        self.sol.send_break()
        return

    def sol_data_received(self, conn, string):
        if (not self.take_input):
            self.nack_count += 1
            return 1
        try:
            self.term.ProcessInput(string)
        except Exception, e:
            import sys
            t, v, b = sys.exc_info()
            sys.excepthook(t, v, b)
            del b
            pass
        return 0
    
    def sol_break_detected(self, conn):
        self.errstr.SetError("Received break")
        return
    
    def sol_bmc_transmit_overrun(self, conn):
        self.errstr.SetError("BMC Transmit Overrun")
        return

    pass
